ALGORITHM_SUBMODULES = [
    "networkx.linalg.algebraicconnectivity",
    "networkx.algorithms.centrality",
    "networkx.algorithms.centrality.load",
    "networkx.algorithms.clique",
    "networkx.algorithms.cluster",
    "networkx.algorithms.components",
    "networkx.algorithms.connectivity",
    "networkx.algorithms.efficiency_measures",
    "networkx.algorithms.cycles",
    "networkx.algorithms.flow",
    "networkx.algorithms.shortest_paths",
    "networkx.algorithms.assortativity",
    "networkx.algorithms.lowest_common_ancestors",
    "networkx.algorithms.wiener",
    "networkx.algorithms.community",
    "networkx.algorithms.coloring",
    "networkx.algorithms.isomorphism",
    "networkx.algorithms.triads",
    "networkx.algorithms.bridges",
    "networkx.algorithms.richclub",
    "networkx.algorithms.matching",
    "networkx.algorithms.dag",
    "networkx.algorithms.tree",
    "networkx.algorithms.approximation",
    "networkx.algorithms.link_analysis",
    "networkx.algorithms.traversal",
    "networkx.algorithms.approximation",
    "networkx.algorithms.link_prediction",
    "networkx.algorithms.vitality",
    "networkx.algorithms.core",
    "networkx.algorithms.distance_measures",
]
